@@ -11,6 +11,7 @@ else:
 import scipy.stats as sps
 import numpy as np
 import itertools
+import warnings
 
 dists = {
     'alpha' : sps.alpha,
@@ -173,23 +174,13 @@ def __get_bounds(function_name = None,steps = 200,*args):
 
 def lognormal(mean, var, steps = 200):
 
-    x = np.linspace(0,1,steps)
+    args = list(args)
+    for i in range(0,len(args)):
+        if args[i].__class__.__name__ != 'Interval':
+            args[i] = Interval(args[i])
 
-    if mean.__class__.__name__ != 'Interval':
-        mean = Interval(mean,mean)
-    if var.__class__.__name__ != 'Interval':
-        var = Interval(var,var)
+    Left, Right, mean, var = __get_bounds('lognormal',steps,*args)
 
-    bound0 = sps.lognorm.ppf(x, mean.left, var.left)
-    bound1 = sps.lognorm.ppf(x, mean.right, var.left)
-    bound2 = sps.lognorm.ppf(x, mean.left, var.right)
-    bound3 = sps.lognorm.ppf(x, mean.right, var.right)
-
-    Left = [min(bound0[i],bound1[i],bound2[i],bound3[i]) for i in range(steps)]
-    Right = [max(bound0[i],bound1[i],bound2[i],bound3[i]) for i in range(steps)]
-
-    Left = np.array(Left)
-    Right = np.array(Right)
     return Pbox(
         Left,
         Right,
@@ -278,13 +269,18 @@ def argus(*args, steps = 200):
           )
 
 def beta(*args, steps = 200):
+    '''
+    Beta distribution
+    '''
     args = list(args)
     for i in range(0,len(args)):
         if args[i].__class__.__name__ != 'Interval':
             args[i] = Interval(args[i])
         if args[i].left == 0:
             args[i].left = 1e-5
-
+        if args[i].right == 0:
+            args[i].right = 1e-5
+            
     Left, Right, mean, var = __get_bounds('beta',steps,*args)
 
     return Pbox(
@@ -1906,8 +1902,8 @@ def trapz(a,b,c,d , steps = 200):
         d = Interval(d)
 
     x = np.linspace(0.0001,0.9999,steps)
-    left = sps.trapz.ppf(x,b.lo()/d.lo(),c.lo()/d.lo(),a.lo(),d.lo()-a.lo())
-    right = sps.trapz.ppf(x,b.hi()/d.hi(),c.hi()/d.hi(),a.hi(),d.hi()-a.hi())
+    left = sps.trapz.ppf(x,*sorted([b.lo()/d.lo(),c.lo()/d.lo(),a.lo(),d.lo()-a.lo()]))
+    right = sps.trapz.ppf(x,*sorted([b.hi()/d.hi(),c.hi()/d.hi(),a.hi(),d.hi()-a.hi()]))
 
     return Pbox(
           left,
@@ -1954,13 +1950,21 @@ def truncexpon(*args, steps = 200):
           var_right  = var.right
           )
 
-def truncnorm(*args, steps = 200):
-    args = list(args)
-    for i in range(0,len(args)):
-        if args[i].__class__.__name__ != 'Interval':
-            args[i] = Interval(args[i])
+def truncnorm(left,right,mean = None,stddev = None, steps = 200):
+    
+    if left.__class__.__name__ != 'Interval':
+        left = Interval(left)
+    if right.__class__.__name__ != 'Interval':
+        right = Interval(right)
+    if mean.__class__.__name__ != 'Interval':
+        mean = Interval(mean)
+    if stddev.__class__.__name__ != 'Interval':
+        stddev = Interval(stddev)
+    
+    a,b = (left - mean)/stddev, (right - mean)/stddev
+    
 
-    Left, Right, mean, var = __get_bounds('truncnorm',steps,*args)
+    Left, Right, mean, var = __get_bounds('truncnorm',steps,a,b,mean,stddev)
 
     return Pbox(
           Left,
@@ -2421,7 +2425,9 @@ def yulesimon(*args, steps = 200):
 
 ### Other distributions
 def KM(k,m,steps = 200):
-    return beta(Interval(k,k+1),Interval(m,m+1),steps = steps)
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        return beta(Interval(k,k+1),Interval(m,m+1),steps = steps)
 
 def KN(k,n,steps = 200):
     return KM(k,n-k,steps=steps)
